@@ -68,11 +68,17 @@ import javax.imageio.spi.ImageWriterSpi;
 import javax.imageio.stream.ImageInputStream;
 import javax.imageio.stream.ImageOutputStream;
 
+import gnu.classpath.Configuration;
+import gnu.classpath.Pointer;
+
 public class GdkPixbufDecoder extends gnu.java.awt.image.ImageDecoder
 {
   static 
   {
-    System.loadLibrary("gtkpeer");
+    if (Configuration.INIT_LOAD_LIBRARY)
+      {
+        System.loadLibrary("gtkpeer");
+      }
 
     initStaticState ();
   }
@@ -94,6 +100,14 @@ public class GdkPixbufDecoder extends gnu.java.awt.image.ImageDecoder
   // the current set of ImageConsumers for this decoder
   Vector curr;
 
+  /**
+   * The pointer to the native pixbuf loader.
+   *
+   * This field is manipulated by native code. Don't change or remove
+   * without adjusting the native code.
+   */
+  private Pointer nativeDecoder;
+
   // interface to GdkPixbuf
   // These native functions should be called with the pixbufLock held.
   native void initState ();
