@@ -1,8 +1,8 @@
 #ifndef _CLASSPATH_INCLUDE_CONFIG_INT_H
 #define _CLASSPATH_INCLUDE_CONFIG_INT_H 1
 #ifndef _GENERATED_STDINT_H
-#define _GENERATED_STDINT_H "classpath 0.97"
-/* generated using gnu compiler gcc (GCC) 4.1.2 (Gentoo 4.1.2 p1.0.2) */
+#define _GENERATED_STDINT_H "classpath 0.97.1"
+/* generated using gnu compiler gcc (GCC) 4.1.2 20061115 (prerelease) (Debian 4.1.1-21) */
 #define _STDINT_HAVE_STDINT_H 1
 #include <stdint.h>
 #endif
