@@ -1,5 +1,5 @@
 /* DataOutputStream.java -- Writes primitive Java datatypes to streams
-   Copyright (C) 1998, 2001, 2003, 2005  Free Software Foundation, Inc.
+   Copyright (C) 1998, 2001, 2003, 2005, 2008  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -62,6 +62,11 @@ public class DataOutputStream extends FilterOutputStream implements DataOutput
    */
   protected int written;
 
+  /**
+   * Utf8 byte buffer, used by writeUTF()
+   */
+  private byte[] buf;
+  
   /**
    * This method initializes an instance of <code>DataOutputStream</code> to
    * write its data to the specified underlying <code>OutputStream</code>
@@ -302,7 +307,7 @@ public class DataOutputStream extends FilterOutputStream implements DataOutput
    *
    * @exception IOException If an error occurs
    *
-   * @see writeInt
+   * @see #writeInt(int)
    * @see DataInput#readFloat
    * @see Float#floatToIntBits
    */
@@ -326,7 +331,7 @@ public class DataOutputStream extends FilterOutputStream implements DataOutput
    *
    * @exception IOException If an error occurs
    *
-   * @see writeLong
+   * @see #writeLong(long)
    * @see DataInput#readDouble
    * @see Double#doubleToLongBits
    */
@@ -363,7 +368,7 @@ public class DataOutputStream extends FilterOutputStream implements DataOutput
    *
    * @exception IOException If an error occurs
    *
-   * @see writeChar
+   * @see #writeChar(char)
    */
   public final void writeChars (String value) throws IOException
   {
@@ -372,6 +377,35 @@ public class DataOutputStream extends FilterOutputStream implements DataOutput
       writeChar (value.charAt(i));
   }
 
+  /**
+   *  Calculate the length, in bytes, of a <code>String</code> in Utf8 format.
+   *  This method is package-private so that <code>ObjectOutputStream</code>
+   *  may use it.  The return type is long so that a long string whose
+   *  Utf8 byte count is 64 bit long may be handled.
+   *
+   *  @param value The <code>String</code> to measure
+   *  @param start String index at which to begin count
+   *  @param sum Starting Utf8 byte count
+   *
+   */
+  long getUTFlength(String value, int start, long sum)
+  {
+    int len = value.length();
+
+    for (int i = start; i < len; ++i)
+      {
+	char c = value.charAt(i);
+	if (c >= '\u0001' && c <= '\u007f')
+	  sum += 1;
+	else if (c == '\u0000' || (c >= '\u0080' && c <= '\u07ff'))
+	  sum += 2;
+	else
+	  sum += 3;
+      }
+
+    return sum;
+  }
+  
   /**
    * This method writes a Java <code>String</code> to the stream in a modified
    * UTF-8 format.  First, two bytes are written to the stream indicating the
@@ -406,49 +440,100 @@ public class DataOutputStream extends FilterOutputStream implements DataOutput
    */
   public final synchronized void writeUTF(String value) throws IOException
   {
-    int len = value.length();
-    int sum = 0;
-
-    for (int i = 0; i < len && sum <= 65535; ++i)
-      {
-	char c = value.charAt(i);
-	if (c >= '\u0001' && c <= '\u007f')
-	  sum += 1;
-	else if (c == '\u0000' || (c >= '\u0080' && c <= '\u07ff'))
-	  sum += 2;
-	else
-	  sum += 3;
-      }
-
-    if (sum > 65535)
+    long l = getUTFlength(value, 0, 0);
+    if (l > 65535)
       throw new UTFDataFormatException ();
+    writeUTFShort(value, (int)l);
+  }
 
+  /**
+   * This method performs the main task of <code>writeUTF</code>.
+   * This method is package-private because ObjectOutputStream uses it.
+   *
+   * @param value The <code>String</code> to write to the output in UTF format
+   *
+   * @param bytelen The UTF-8 byte length of the <code>String</code>. When
+   * this method is called, the expected byte length must have been calculated
+   * by <code>getUTFlength</code>.
+   *
+   * @exception IOException If an error occurs
+   *
+   * @see DataInput#readUTF
+   */
+  final synchronized void writeUTFShort(String value, int bytelen)
+    throws IOException
+  {
+    writeShort(bytelen);
+    writeUTFBytes(value);
+  }
+
+  /**
+   * This method is similar to <code>writeUTF</code>, but it writes the
+   * UTF-8 byte length in 64 bits.
+   * This method is not public but <code>ObjectOutputStream</code> uses it.
+   *
+   * @param value The <code>String</code> to write to the output in UTF format
+   *
+   * @param bytelen The UTF-8 byte length of the <code>String</code>. When
+   * this method is called, the expected byte length must have been calculated
+   * by <code>getUTFlength</code>.
+   *
+   * @exception IOException If an error occurs
+   *
+   */
+  final synchronized void writeUTFLong(String value, long bytelen)
+    throws IOException
+  {
+    writeLong(bytelen);
+    writeUTFBytes(value);
+  }
+
+  /**
+   * This method performes the main task of <code>writeUTF</code> and
+   * <code>WriteUTFLong</code>, which is to write the UTF-8 byte
+   * sequence to the output.
+   *
+   * @param value The <code>String</code> to write to the output in UTF format
+   *
+   * @exception IOException If an error occurs
+   *
+   */
+  private final synchronized void writeUTFBytes(String value)
+    throws IOException
+  {
+    int len = value.length();
+    int i = 0;
     int pos = 0;
-    byte[] buf = new byte[sum];
 
-    for (int i = 0; i < len; ++i)
-      {
-	char c = value.charAt(i);
-	if (c >= '\u0001' && c <= '\u007f')
-          buf[pos++] = (byte) c;
-	else if (c == '\u0000' || (c >= '\u0080' && c <= '\u07ff'))
-	  {
-	    buf[pos++] = (byte) (0xc0 | (0x1f & (c >> 6)));
-	    buf[pos++] = (byte) (0x80 | (0x3f & c));
-	  }
-	else
-	  {
-	    // JSL says the first byte should be or'd with 0xc0, but
-	    // that is a typo.  Unicode says 0xe0, and that is what is
-	    // consistent with DataInputStream.
-	    buf[pos++] = (byte) (0xe0 | (0x0f & (c >> 12)));
-	    buf[pos++] = (byte) (0x80 | (0x3f & (c >> 6)));
-	    buf[pos++] = (byte) (0x80 | (0x3f & c));
-	  }
-      }
+    if (buf == null)
+      buf = new byte[512];
     
-    writeShort (sum);
-    write(buf, 0, sum);
+    do
+      {
+	while (i < len && pos < buf.length - 3)
+	  {
+	    char c = value.charAt(i++);
+	    if (c >= '\u0001' && c <= '\u007f')
+              buf[pos++] = (byte) c;
+	    else if (c == '\u0000' || (c >= '\u0080' && c <= '\u07ff'))
+	      {
+		buf[pos++] = (byte) (0xc0 | (0x1f & (c >> 6)));
+		buf[pos++] = (byte) (0x80 | (0x3f & c));
+	      }
+	    else
+	      {
+		// JSL says the first byte should be or'd with 0xc0, but
+		// that is a typo.  Unicode says 0xe0, and that is what is
+		// consistent with DataInputStream.
+		buf[pos++] = (byte) (0xe0 | (0x0f & (c >> 12)));
+		buf[pos++] = (byte) (0x80 | (0x3f & (c >> 6)));
+		buf[pos++] = (byte) (0x80 | (0x3f & c));
+	      }
+	  }
+	write(buf, 0, pos);
+	pos = 0;
+     }
+    while (i < len);
   }
 
 } // class DataOutputStream
