@@ -1,5 +1,5 @@
 /* Locale.java -- i18n locales
-   Copyright (C) 1998, 1999, 2001, 2002, 2005  Free Software Foundation, Inc.
+   Copyright (C) 1998, 1999, 2001, 2002, 2005, 2006  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -39,12 +39,15 @@ exception statement from your version. */
 package java.util;
 
 import gnu.classpath.SystemProperties;
+import gnu.java.locale.LocaleHelper;
 
 import java.io.IOException;
 import java.io.ObjectInputStream;
 import java.io.ObjectOutputStream;
 import java.io.Serializable;
 
+import java.util.spi.LocaleNameProvider;
+
 /**
  * Locales represent a specific country and culture. Classes which can be
  * passed a Locale object tailor their information for a given locale. For
@@ -65,11 +68,12 @@ import java.io.Serializable;
  * be separated by an underscore (U+005F).
  *
  * <p>The default locale is determined by the values of the system properties
- * user.language, user.region, and user.variant, defaulting to "en". Note that
- * the locale does NOT contain the conversion and formatting capabilities (for
- * that, use ResourceBundle and java.text). Rather, it is an immutable tag
- * object for identifying a given locale, which is referenced by these other
- * classes when they must make locale-dependent decisions.
+ * user.language, user.country (or user.region), and user.variant, defaulting
+ * to "en_US". Note that the locale does NOT contain the conversion and
+ * formatting capabilities (for that, use ResourceBundle and java.text).
+ * Rather, it is an immutable tag object for identifying a given locale, which
+ * is referenced by these other classes when they must make locale-dependent
+ * decisions.
  *
  * @see ResourceBundle
  * @see java.text.Format
@@ -159,6 +163,11 @@ public final class Locale implements Serializable, Cloneable
   /** Locale which represents the French speaking portion of Canada. */
   public static final Locale CANADA_FRENCH = getLocale("fr", "CA");
 
+  /** The root locale, used as the base case in lookups by
+   *  locale-sensitive operations.
+   */
+  public static final Locale ROOT = new Locale("","","");
+
   /**
    * Compatible with JDK 1.1+.
    */
@@ -169,28 +178,28 @@ public final class Locale implements Serializable, Cloneable
    *
    * @serial the languange, possibly ""
    */
-  private String language;
+  private final String language;
 
   /**
    * The country code, as returned by getCountry().
    *
    * @serial the country, possibly ""
    */
-  private String country;
+  private final String country;
 
   /**
    * The variant code, as returned by getVariant().
    *
    * @serial the variant, possibly ""
    */
-  private String variant;
+  private final String variant;
 
   /**
    * This is the cached hashcode. When writing to stream, we write -1.
    *
    * @serial should be -1 in serial streams
    */
-  private transient int hashcode;
+  private int hashcode;
 
   /**
    * Array storing all available locales.
@@ -209,10 +218,18 @@ public final class Locale implements Serializable, Cloneable
    * null. Note the logic in the main constructor, to detect when
    * bootstrapping has completed.
    */
-  private static Locale defaultLocale =
-    getLocale(SystemProperties.getProperty("user.language", "en"),
-              SystemProperties.getProperty("user.region", ""),
-              SystemProperties.getProperty("user.variant", ""));
+  private static Locale defaultLocale;
+
+  static {
+    String language = SystemProperties.getProperty("user.language", "en");
+    String country  = SystemProperties.getProperty("user.country", "US");
+    String region   = SystemProperties.getProperty("user.region", null);
+    String variant  = SystemProperties.getProperty("user.variant", "");
+
+    defaultLocale = getLocale(language,
+                              (region != null) ? region : country,
+                              variant);
+  }
 
   /**
    * Array storing all the available two-letter ISO639 languages.
@@ -236,38 +253,38 @@ public final class Locale implements Serializable, Cloneable
   }
   
   /**
-   * Retrieves the locale with the specified language and region
+   * Retrieves the locale with the specified language and country
    * from the cache.
    *
    * @param language the language of the locale to retrieve.
-   * @param region the region of the locale to retrieve.
+   * @param country the country of the locale to retrieve.
    * @return the locale.
    */ 
-  private static Locale getLocale(String language, String region)
+  private static Locale getLocale(String language, String country)
   {
-    return getLocale(language, region, "");
+    return getLocale(language, country, "");
   }
   
   /**
-   * Retrieves the locale with the specified language, region
+   * Retrieves the locale with the specified language, country
    * and variant from the cache.
    *
    * @param language the language of the locale to retrieve.
-   * @param region the region of the locale to retrieve.
+   * @param country the country of the locale to retrieve.
    * @param variant the variant of the locale to retrieve.
    * @return the locale.
    */ 
-  private static Locale getLocale(String language, String region, String variant)
+  private static Locale getLocale(String language, String country, String variant)
   {
     if (localeMap == null)
       localeMap = new HashMap(256);
 
-    String name = language + "_" + region + "_" + variant;
+    String name = language + "_" + country + "_" + variant;
     Locale locale = (Locale) localeMap.get(name);
 
     if (locale == null)
       {
-	locale = new Locale(language, region, variant);
+	locale = new Locale(language, country, variant);
 	localeMap.put(name, locale);
       }
 
@@ -307,13 +324,12 @@ public final class Locale implements Serializable, Cloneable
     // default locale.
     if (defaultLocale != null)
       {
-        language = convertLanguage(language).intern();
-        country = country.toUpperCase().intern();
-        variant = variant.intern();
+        language = convertLanguage(language);
+        country = country.toUpperCase();
       }
-    this.language = language;
-    this.country = country;
-    this.variant = variant;
+    this.language = language.intern();
+    this.country = country.intern();
+    this.variant = variant.intern();
     hashcode = language.hashCode() ^ country.hashCode() ^ variant.hashCode();
   }
 
@@ -384,33 +400,33 @@ public final class Locale implements Serializable, Cloneable
   {
     if (availableLocales == null)
       {
-	String[] localeNames = LocaleData.localeNames;
-        availableLocales = new Locale[localeNames.length];
+        int len = LocaleHelper.getLocaleCount();
+        availableLocales = new Locale[len];
 
-        for (int i = 0; i < localeNames.length; i++)
+        for (int i = 0; i < len; i++)
           {
             String language;
-            String region = "";
+            String country = "";
             String variant = "";
-            String name = localeNames[i];
+            String name = LocaleHelper.getLocaleName(i);
 
             language = name.substring(0, 2);
 
             if (name.length() > 2)
-              region = name.substring(3);
+              country = name.substring(3);
 
-	    int index = region.indexOf("_");
+	    int index = country.indexOf("_");
 	    if (index > 0)
 	      {
-		variant = region.substring(index + 1);
-		region = region.substring(0, index - 1);
+		variant = country.substring(index + 1);
+		country = country.substring(0, index - 1);
 	      }
 
-            availableLocales[i] = getLocale(language, region, variant);
+            availableLocales[i] = getLocale(language, country, variant);
           }
       }
     
-    return availableLocales;
+    return (Locale[]) availableLocales.clone();
   }
 
   /**
@@ -426,7 +442,7 @@ public final class Locale implements Serializable, Cloneable
 	countryCache = getISOStrings("territories");
       }
 
-    return countryCache;
+    return (String[]) countryCache.clone();
   }
 
   /**
@@ -441,7 +457,7 @@ public final class Locale implements Serializable, Cloneable
       {
 	languageCache = getISOStrings("languages");
       }
-    return languageCache;
+    return (String[]) languageCache.clone();
   }
 
   /**
@@ -664,6 +680,8 @@ public final class Locale implements Serializable, Cloneable
    */
   public String getDisplayLanguage(Locale inLocale)
   {
+    if (language.isEmpty())
+      return "";
     try
       {
 	ResourceBundle res =
@@ -675,8 +693,27 @@ public final class Locale implements Serializable, Cloneable
       }
     catch (MissingResourceException e)
       {
-	return language;
+	/* This means runtime support for the locale
+	 * is not available, so we check providers. */
       }
+    for (LocaleNameProvider p :
+	   ServiceLoader.load(LocaleNameProvider.class))
+      {
+	for (Locale loc : p.getAvailableLocales())
+	  {
+	    if (loc.equals(inLocale))
+	      {
+		String locLang = p.getDisplayLanguage(language,
+						      inLocale);
+		if (locLang != null)
+		  return locLang;
+		break;
+	      }
+	  }
+      }
+    if (inLocale.equals(Locale.ROOT)) // Base case
+      return language;
+    return getDisplayLanguage(LocaleHelper.getFallbackLocale(inLocale));
   }
 
   /**
@@ -722,6 +759,8 @@ public final class Locale implements Serializable, Cloneable
    */
   public String getDisplayCountry(Locale inLocale)
   {
+    if (country.isEmpty())
+      return "";
     try
       {
         ResourceBundle res =
@@ -733,8 +772,27 @@ public final class Locale implements Serializable, Cloneable
       }
     catch (MissingResourceException e)
       {
-        return country;
+	/* This means runtime support for the locale
+	 * is not available, so we check providers. */
       }
+    for (LocaleNameProvider p :
+	   ServiceLoader.load(LocaleNameProvider.class))
+      {
+	for (Locale loc : p.getAvailableLocales())
+	  {
+	    if (loc.equals(inLocale))
+	      {
+		String locCountry = p.getDisplayCountry(country,
+							inLocale);
+		if (locCountry != null)
+		  return locCountry;
+		break;
+	      }
+	  }
+      }
+    if (inLocale.equals(Locale.ROOT)) // Base case
+      return country;
+    return getDisplayCountry(LocaleHelper.getFallbackLocale(inLocale));
   }
 
   /**
@@ -781,6 +839,8 @@ public final class Locale implements Serializable, Cloneable
    */
   public String getDisplayVariant(Locale inLocale)
   {
+    if (variant.isEmpty())
+      return "";
     try
       {
         ResourceBundle res =
@@ -792,8 +852,27 @@ public final class Locale implements Serializable, Cloneable
       }
     catch (MissingResourceException e)
       {
-        return variant;
+	/* This means runtime support for the locale
+	 * is not available, so we check providers. */
       }
+    for (LocaleNameProvider p :
+	   ServiceLoader.load(LocaleNameProvider.class))
+      {
+	for (Locale loc : p.getAvailableLocales())
+	  {
+	    if (loc.equals(inLocale))
+	      {
+		String locVar = p.getDisplayVariant(variant,
+						    inLocale);
+		if (locVar != null)
+		  return locVar;
+		break;
+	      }
+	  }
+      }
+    if (inLocale.equals(Locale.ROOT)) // Base case
+      return country;
+    return getDisplayVariant(LocaleHelper.getFallbackLocale(inLocale));
   }
 
   /**
@@ -907,15 +986,15 @@ public final class Locale implements Serializable, Cloneable
       return false;
     Locale l = (Locale) obj;
 
-    return (language == l.language
-            && country == l.country
+    return (language == l.language 
+            && country == l.country 
             && variant == l.variant);
   }
 
   /**
    * Write the locale to an object stream.
    *
-   * @param output the stream to write to
+   * @param s the stream to write to
    * @throws IOException if the write fails
    * @serialData The first three fields are Strings representing language,
    *             country, and variant. The fourth field is a placeholder for 
@@ -925,17 +1004,15 @@ public final class Locale implements Serializable, Cloneable
   private void writeObject(ObjectOutputStream s)
     throws IOException
   {
-    s.writeObject(language);
-    s.writeObject(country);
-    s.writeObject(variant);
-    // Hashcode field is always written as -1.
-    s.writeInt(-1);
+    ObjectOutputStream.PutField fields = s.putFields();
+    fields.put("hashcode", -1);
+    s.defaultWriteObject();
   }
 
   /**
    * Reads a locale from the input stream.
    *
-   * @param input the stream to read from
+   * @param s the stream to read from
    * @throws IOException if reading fails
    * @throws ClassNotFoundException if reading fails
    * @serialData the hashCode is always invalid and must be recomputed
@@ -943,10 +1020,7 @@ public final class Locale implements Serializable, Cloneable
   private void readObject(ObjectInputStream s)
     throws IOException, ClassNotFoundException
   {
-    language = ((String) s.readObject()).intern();
-    country = ((String) s.readObject()).intern();
-    variant = ((String) s.readObject()).intern();
-    // Recompute hashcode.
+    s.defaultReadObject();
     hashcode = language.hashCode() ^ country.hashCode() ^ variant.hashCode();
   }
 } // class Locale
