@@ -1,5 +1,5 @@
 /* DeflaterEngine.java --
-   Copyright (C) 2001, 2004  Free Software Foundation, Inc.
+   Copyright (C) 2001, 2004, 2005  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -92,7 +92,7 @@ class DeflaterEngine implements DeflaterConstants
   private byte[] inputBuf;
 
   /** The total bytes of input read. */
-  private int totalIn;
+  private long totalIn;
 
   /** The offset into inputBuf, where input data starts. */
   private int inputOff;
@@ -163,7 +163,7 @@ class DeflaterEngine implements DeflaterConstants
     return chksum;
   }
 
-  public final int getTotalIn()
+  public final long getTotalIn()
   {
     return totalIn;
   }
@@ -377,7 +377,8 @@ class DeflaterEngine implements DeflaterConstants
 	     && window[++scan] == window[++match]
 	     && window[++scan] == window[++match]
 	     && window[++scan] == window[++match]
-	     && scan < strend);
+	     && scan < strend)
+        ;
 
       if (scan > best_end) {
 //  	if (DeflaterConstants.DEBUGGING && ins_h == 0)
@@ -497,7 +498,7 @@ class DeflaterEngine implements DeflaterConstants
 		      throw new InternalError();
 		  }
 	      }
-	    huffman.tallyDist(strstart - matchStart, matchLen);
+	    boolean full = huffman.tallyDist(strstart - matchStart, matchLen);
 	    
 	    lookahead -= matchLen;
 	    if (matchLen <= max_lazy && lookahead >= MIN_MATCH)
@@ -516,7 +517,8 @@ class DeflaterEngine implements DeflaterConstants
 		  updateHash();
 	      }
 	    matchLen = MIN_MATCH - 1;
-	    continue;
+	    if (!full)
+	      continue;
 	  }
 	else
 	  {
