@@ -1,5 +1,5 @@
 /* JFrame.java --
-   Copyright (C) 2002, 2004, 2005  Free Software Foundation, Inc.
+   Copyright (C) 2002, 2004, 2005, 2006,  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -76,7 +76,7 @@ public class JFrame extends Frame
     /**
      * Creates a new instance of <code>AccessibleJFrame</code>.
      */
-    public AccessibleJFrame()
+    protected AccessibleJFrame()
     {
       super();
       // Nothing to do here.
@@ -86,6 +86,8 @@ public class JFrame extends Frame
   /**
    * A flag for {@link #setDefaultCloseOperation(int)}, indicating that the
    * application should be exited, when this <code>JFrame</code> is closed.
+   * Note that in version 1.4, the equivalent constant has been added to
+   * {@link WindowConstants}.
    *
    * @since 1.3
    */
@@ -93,7 +95,7 @@ public class JFrame extends Frame
 
   private static final long serialVersionUID = -3362141868504252139L;
   private static boolean defaultLookAndFeelDecorated;
-  private int close_action = HIDE_ON_CLOSE;
+  private int closeAction = HIDE_ON_CLOSE;
   protected AccessibleContext accessibleContext;
   protected JRootPane rootPane;
   
@@ -103,18 +105,19 @@ public class JFrame extends Frame
   protected boolean rootPaneCheckingEnabled = false;
 
   /**
-   * Tells us if we're in the initialization stage.
-   * If so, adds go to top-level Container, otherwise they go
-   * to the content pane for this container.
+   * Creates a new frame with an empty string for the title.
    */
-  private boolean initStageDone = false;
-
   public JFrame()
   {
-    super("JFrame");
+    super("");
     frameInit();
   }
 
+  /**
+   * Creates a new <code>JFrame</code> with the specified title.
+   * 
+   * @param title  the frame title (<code>null</code> permitted).
+   */
   public JFrame(String title)
   {
     super(title);
@@ -154,11 +157,25 @@ public class JFrame extends Frame
 
   protected void frameInit()
   {
-    super.setLayout(new BorderLayout(1, 1));
+    // We need to explicitly enable events here so that our processKeyEvent()
+    // and processWindowEvent() gets called.
+    enableEvents(AWTEvent.WINDOW_EVENT_MASK | AWTEvent.KEY_EVENT_MASK);
+
+    super.setLayout(new BorderLayout());
+    setBackground(UIManager.getDefaults().getColor("control"));
     enableEvents(AWTEvent.WINDOW_EVENT_MASK);
     getRootPane(); // will do set/create
+
+    // Setup the defaultLookAndFeelDecoration if requested.
+    if (isDefaultLookAndFeelDecorated()
+        && UIManager.getLookAndFeel().getSupportsWindowDecorations())
+      {
+        setUndecorated(true);
+        getRootPane().setWindowDecorationStyle(JRootPane.FRAME);
+      }
+
     // We're now done the init stage.
-    initStageDone = true;
+    setRootPaneCheckingEnabled(true);
   }
 
   public Dimension getPreferredSize()
@@ -180,13 +197,8 @@ public class JFrame extends Frame
   {
     // Check if we're in initialization stage.  If so, call super.setLayout
     // otherwise, valid calls go to the content pane.
-    if (initStageDone)
-      {
-        if (isRootPaneCheckingEnabled())
-          throw new Error("Cannot set layout. Use getContentPane().setLayout()"
-                           + " instead.");
-        getContentPane().setLayout(manager);
-      }
+    if (isRootPaneCheckingEnabled())
+      getContentPane().setLayout(manager);
     else
       super.setLayout(manager);
   }
@@ -246,15 +258,10 @@ public class JFrame extends Frame
   {
     // If we're adding in the initialization stage use super.add.
     // Otherwise pass the add onto the content pane.
-    if (!initStageDone)
-      super.addImpl(comp, constraints, index);
+    if (isRootPaneCheckingEnabled() && comp != rootPane)
+      getContentPane().add(comp,constraints,index);
     else
-      {
-        if (isRootPaneCheckingEnabled())
-          throw new Error("rootPaneChecking is enabled - adding components "
-                           + "disallowed.");
-        getContentPane().add(comp,constraints,index);
-      }
+      super.addImpl(comp, constraints, index);
   }
 
   public void remove(Component comp)
@@ -297,6 +304,12 @@ public class JFrame extends Frame
     return defaultLookAndFeelDecorated;
   }
 
+  /**
+   * Returns the object that provides accessibility features for this 
+   * <code>JFrame</code>.
+   *
+   * @return The accessible context (an instance of {@link AccessibleJFrame}).
+   */
   public AccessibleContext getAccessibleContext()
   {
     if (accessibleContext == null)
@@ -304,67 +317,80 @@ public class JFrame extends Frame
     return accessibleContext;
   }
 
+  /**
+   * Returns a code for the default operation when the frame is closed.  The
+   * default value is {@link WindowConstants#HIDE_ON_CLOSE}.
+   * 
+   * @return One of: {@link WindowConstants#DO_NOTHING_ON_CLOSE},
+   *     {@link WindowConstants#HIDE_ON_CLOSE}, 
+   *     {@link WindowConstants#DISPOSE_ON_CLOSE}, {@link #EXIT_ON_CLOSE}.
+   * 
+   * @see #setDefaultCloseOperation(int)
+   */
   public int getDefaultCloseOperation()
   {
-    return close_action;
+    return closeAction;
   }
 
+  /**
+   * Returns a string describing the attributes for the <code>JFrame</code>,
+   * for use in debugging.  The return value is guaranteed to be 
+   * non-<code>null</code>, but the format may vary between implementations.
+   * 
+   * @return A string describing the attributes of the <code>JFrame</code>.
+   */
   protected String paramString()
   {
-    return "JFrame";
+    StringBuffer sb = new StringBuffer(super.paramString());
+    sb.append(",defaultCloseOperation=");
+    sb.append(SwingUtilities.convertWindowConstantToString(
+        getDefaultCloseOperation()));
+    sb.append(",rootPane=");
+    if (rootPane != null)
+      sb.append(rootPane);
+    sb.append(",rootPaneCheckingEnabled=").append(rootPaneCheckingEnabled);
+    return sb.toString();
   }
 
   protected void processWindowEvent(WindowEvent e)
   {
     super.processWindowEvent(e);
-    switch (e.getID())
+    if (e.getID() == WindowEvent.WINDOW_CLOSING)
       {
-      case WindowEvent.WINDOW_CLOSING:
-        {
-	  switch (close_action)
-	    {
-	    case EXIT_ON_CLOSE:
-	      {
-		System.exit(0);
-		break;
-	      }
-	    case DISPOSE_ON_CLOSE:
-	      {
-		dispose();
-		break;
-	      }
-	    case HIDE_ON_CLOSE:
-	      {
-		setVisible(false);
-		break;
-	      }
-	    case DO_NOTHING_ON_CLOSE:
-	      break;
-	    }
-	  break;
-        }
-      case WindowEvent.WINDOW_CLOSED:
-      case WindowEvent.WINDOW_OPENED:
-      case WindowEvent.WINDOW_ICONIFIED:
-      case WindowEvent.WINDOW_DEICONIFIED:
-      case WindowEvent.WINDOW_ACTIVATED:
-      case WindowEvent.WINDOW_DEACTIVATED:
-	break;
+        switch (closeAction)
+	  {
+	  case EXIT_ON_CLOSE:
+	    System.exit(0);
+	    break;
+	  case DISPOSE_ON_CLOSE:
+	    dispose();
+	    break;
+	  case HIDE_ON_CLOSE:
+	    setVisible(false);
+	    break;
+	  case DO_NOTHING_ON_CLOSE:
+	    break;
+	  }
       }
   }
 
   /**
-   * Defines what happens when this frame is closed. Can be one off
-   * <code>EXIT_ON_CLOSE</code>,
-   * <code>DISPOSE_ON_CLOSE</code>,
-   * <code>HIDE_ON_CLOSE</code> or
-   * <code>DO_NOTHING_ON_CLOSE</code>.
-   * The default is <code>HIDE_ON_CLOSE</code>.
-   * When <code>EXIT_ON_CLOSE</code> is specified this method calls
+   * Sets the default operation that is performed when this frame is closed.
+   * The default is <code>HIDE_ON_CLOSE</code>.  When 
+   * <code>EXIT_ON_CLOSE</code> is specified this method calls
    * <code>SecurityManager.checkExit(0)</code> which might throw a
-   * <code>SecurityException</code>. When the specified operation is
-   * not one of the above a <code>IllegalArgumentException</code> is
-   * thrown.
+   * <code>SecurityException</code>.
+   * 
+   * @param operation  a code for the operation (one of: 
+   *     {@link WindowConstants#DO_NOTHING_ON_CLOSE}, 
+   *     {@link WindowConstants#HIDE_ON_CLOSE}, 
+   *     {@link WindowConstants#DISPOSE_ON_CLOSE} and 
+   *     {@link WindowConstants#EXIT_ON_CLOSE}).
+   * 
+   * @throws IllegalArgumentException if <code>operation</code> is not one of
+   *     the specified codes.
+   * 
+   * @see #getDefaultCloseOperation()
    */
   public void setDefaultCloseOperation(int operation)
   {
@@ -374,8 +400,9 @@ public class JFrame extends Frame
 
     if (operation != EXIT_ON_CLOSE && operation != DISPOSE_ON_CLOSE
         && operation != HIDE_ON_CLOSE && operation != DO_NOTHING_ON_CLOSE)
-      throw new IllegalArgumentException("defaultCloseOperation must be EXIT_ON_CLOSE, HIDE_ON_CLOSE, DISPOSE_ON_CLOSE, or DO_NOTHING_ON_CLOSE");
+      throw new IllegalArgumentException("operation must be EXIT_ON_CLOSE, " 
+          + "HIDE_ON_CLOSE, DISPOSE_ON_CLOSE, or DO_NOTHING_ON_CLOSE");
 
-    close_action = operation;
+    closeAction = operation;
   }
 }
